from gym.spaces import Box
import numpy as np
from . import convert_box
import lycon

def check_param(obs_space, resize):
    xsize,ysize,linear_interp = resize
    assert all(isinstance(ds, int) and ds > 0 for ds in [xsize,ysize]), "resize x and y sizes must be integers greater than zero."
    assert isinstance(linear_interp, bool), "resize linear_interp parameter must be bool."
    assert len(obs_space.shape) == 3 or len(obs_space.shape) == 2

def change_obs_space(obs_space, param):
    return convert_box(lambda obs:change_observation(obs, obs_space, param), obs_space)

def change_observation(obs, obs_space, resize):
    xsize,ysize,linear_interp = resize
    if len(obs.shape) == 2:
        obs = obs.reshape(obs.shape+(1,))
    interp_method = lycon.Interpolation.LINEAR if linear_interp else lycon.Interpolation.AREA
    obs = lycon.resize(obs, width=xsize, height=ysize, interpolation=interp_method)
    if len(obs_space.shape) == 2:
        obs = obs.reshape(obs.shape[:2])
    return obs



import gym
import importlib

__version__ = "1.1.1"

class WrapperFactory:
    def __init__(self, wrapper_name):
        self.wrapper_name = wrapper_name

    def __call__(self, env, *args, **kwargs):
        if isinstance(env, gym.Env):
            from . import gym_wrappers
            wrap_class = getattr(gym_wrappers, self.wrapper_name)
            return wrap_class(env, *args, **kwargs)
        else:
            from . import aec_wrappers
            from pettingzoo import AECEnv
            assert isinstance(env, AECEnv), "environment must either be a gym environment or a pettingzoo environment"
            wrap_class = getattr(aec_wrappers, self.wrapper_name)
            return wrap_class(env, *args, **kwargs)

color_reduction = WrapperFactory("color_reduction")
resize = WrapperFactory("resize")
dtype = WrapperFactory("dtype")
flatten = WrapperFactory("flatten")
frame_stack = WrapperFactory("frame_stack")
normalize_obs = WrapperFactory("normalize_obs")
reshape = WrapperFactory("reshape")
clip_reward = WrapperFactory("clip_reward")
action_lambda = WrapperFactory("action_lambda")
clip_actions = WrapperFactory("clip_actions")
observation_lambda = WrapperFactory("observation_lambda")
reward_lambda = WrapperFactory("reward_lambda")
frame_skip = WrapperFactory("frame_skip")
sticky_actions = WrapperFactory("sticky_actions")
delay_observations = WrapperFactory("delay_observations")

from .aec_wrappers import agent_indicator, pad_action_space, pad_observations
from .vector_constructors import gym_vec_env, stable_baselines_vec_env, stable_baselines3_vec_env



from supersuit import gym_vec_env
import gym
import numpy as np

def test_vec_env_args():
    env = gym.make("Acrobot-v1")
    num_envs = 8
    vec_env = gym_vec_env(env, num_envs)
    vec_env.reset()
    obs, rew, dones, infos = vec_env.step([0]+[1]*(vec_env.num_envs-1))
    assert not np.any(np.equal(obs[0], obs[1]))

def test_all_vec_env_fns():
    num_envs = 8
    env = gym.make("Acrobot-v1")
    vec_env = gym_vec_env(env, num_envs, False)
    vec_env = gym_vec_env(env, num_envs, True)



import numpy as np
import copy
from gym.spaces import Box
from gym import spaces
import warnings
from pettingzoo.utils.wrappers import AgentIterWrapper as PettingzooWrap


class BaseWrapper(PettingzooWrap):

    metadata = {'render.modes': ['human']}

    def __init__(self, env):
        '''
        Creates a wrapper around `env`. Extend this class to create changes to the space.
        '''
        super().__init__(env)

        self._check_wrapper_params()

        self._modify_spaces()

    def _check_wrapper_params(self):
        raise NotImplementedError()

    def _modify_spaces(self):
        raise NotImplementedError()

    def _modify_action(self, agent, action):
        raise NotImplementedError()

    def _modify_observation(self, agent, observation):
        raise NotImplementedError()

    def _update_step(self, agent, observation):
        raise NotImplementedError()

    def reset(self, observe=True):
        observation = super().reset(observe)
        agent = self.env.agent_selection

        self._update_step(agent,observation)
        if observe:
            observation = self._modify_observation(agent,observation)
            return observation
        else:
            return None

    def observe(self, agent):
        obs = super().observe(agent)
        observation = self._modify_observation(agent, obs)
        return observation

    def step(self, action, observe=True):
        agent = self.env.agent_selection
        cur_act_space = self.action_spaces[agent]
        assert not isinstance(cur_act_space,Box) or cur_act_space.shape == action.shape, "the shape of the action {} is not equal to the shape of the action space {}".format(action.shape,cur_act_space.shape)
        action = self._modify_action(agent, action)
        next_obs = super().step(action, observe=observe)

        new_agent = self.env.agent_selection

        self._update_step(new_agent,next_obs)

        if observe:
            next_obs = self._modify_observation(new_agent,next_obs)
            return next_obs
        else:
            return None
